using System;
using Microsoft.SPOT;
using System.Threading;
using Microsoft.SPOT.Hardware;
using GHIElectronics.NETMF.IO;
using GHIElectronics.NETMF.FEZ;
using GHIElectronics.NETMF.Hardware;


class AccuTest
{

    private const int I2C_BUS_FREQUENCY = 20;           // the frequency the I2C bus should be using
    private int statusWord = 0;




    public void sendStatusWord()
    {

        byte[] byteToSend = new byte[1];
        byteToSend[0] = (byte)statusWord;
        i2cWrite(0x20, byteToSend);
    }



    public void closeAllChannels()
    {
        for (int i = 0; i < 4; i++)
        {
            closeAccuChannel(i);
        }
    }


    public void openAllChannels()
    {
        for (int i = 0; i < 4; i++)
        {
            openAccuChannel(i);
        }
    }



    public void closeAccuChannel(int channel)
    {
        // EEN KANAAL IS ACTIEF ALS DE WAARDE 0 NAAR DAT KANAAL IS GESCHREVEN. 
        switch (channel)
        {
            case 0:
                statusWord = (statusWord & 0xFE);
                break;
            case 1:
                statusWord = (statusWord & 0xFD);
                break;
            case 2:
                statusWord = (statusWord & 0xFB);
                break;
            case 3:
                statusWord = (statusWord & 0xF7);
                break;
        }

        sendStatusWord();
    }


    public void openAccuChannel(int channel)
    {
        // EEN KANAAL IS ACTIEF ALS DE WAARDE F NAAR DAT KANAAL IS GESCHREVEN. 
        // 00000000
        switch (channel)
        {
            case 0:
                statusWord = (statusWord | 0x01);
                break;
            case 1:
                statusWord = (statusWord | 0x02);
                break;
            case 2:
                statusWord = (statusWord | 0x04);
                break;
            case 3:
                statusWord = (statusWord | 0x08);
                break;
        }

        sendStatusWord();
    }


    public void selectMonitoredChannel(int channel)
    {
        statusWord = (statusWord & 0x0F);   // clear the first part of the dataword.
        switch (channel)
        {
            case 0: // 000
                statusWord = (statusWord | 0x00);
                break;
            case 1: // 001
                statusWord = (statusWord | 0x40);   // 00010000
                break;
            case 2: // 010
                statusWord = (statusWord | 0x20);
                break;
            case 3: // 011
                statusWord = (statusWord | 0x60);  // 00110000
                break;
        }

        sendStatusWord();
    }












    /// <summary>
    /// i2cWrite will write the specified data to the chip with the given address.
    /// </summary>
    /// <param name="address">The address of the I2C chip to write to.</param>
    /// <param name="value">A byte array containing all the bytes to send to the chip.</param>
    public static void i2cWrite(int address, byte[] value)
    {
        //create I2C object
        I2CDevice.Configuration con = new I2CDevice.Configuration((ushort)address, I2C_BUS_FREQUENCY);
        I2CDevice MyI2C = new I2CDevice(con);

        // prepare the actions
        I2CDevice.I2CTransaction[] xActions = new I2CDevice.I2CTransaction[1];
        xActions[0] = I2CDevice.CreateWriteTransaction(value);

        // Generate an error when the communication fails.
        if (MyI2C.Execute(xActions, 1000) == 0)
        {   // an error occured
        }
        else
        {
        }

        // Close the I2C bus
        MyI2C.Dispose();
    }

}
