﻿/*****************************************************************************************************************
 * FEZ Battery Voltage Tester
 * 
 * Copyright (c) 2011 - Vanderhaegen Bart
 * 
 * 
 * This testing device is capable to test the voltage of 1.5 V AA batteries. It will both log the output voltages
 * with and without load, over a period of 8 hours. This testing device has no charging possibilities so you have
 * to load fully charged batteries into the device.
 * 
 * The results will be written to the microSD card, in .CSV format. These files can easily be opened with MS Excel.
 * Each battery will get two files: one with the data for the batteries with a load connected to it (normal 
 * situation), and one with the data for the batteries without load.
 * 
 * 
 * Usage:
 * 1) Once you power the FEZ the system goes into standby mode. A green LED will flash to indicate the program is
 * ready to begin. 
 * 2) Now you have to press the red button to start the program. The program will check if an SD card is inserted.
 * So please insert the SD card before you push the button. Otherwise pushing will have no effect.
 * 3) The program will continue for 8 hours. This will be indicated with a flashing green LED. The frequency of
 * the flashing LED is an indication for the remaining time: it will start flashing faster as the program reaches
 * the end of execution.
 * 4) At the end the green LED will burn continiously. It is now safe to take the SD card and put it into a card
 * reader to analyze the data.
 * 5) Once you reinserted the SD card the program will be ready to begin a new cycle. Push the RED button to start
 * the program all over again.
 * 
 * 
 * Principle:
 * The batteries will be discharged with a current of +/- 150 mA. The discharge is computer-controlled: the computer
 * can activate/disactivate the discharge process. Doing so it can characterize your batteries.
 * 
 * 
 * Output Files:
 * The output files are written as .CSV files. The filename contains the datestamp, the number of the channel and
 * the type of test (loaded/unloaded).
 * 
 * 
 * Hardware Connections:
 * - Di3 : SCL
 * - Di2 : SDA
 * - An2 : analog input
 * - Di1 : Status LED
 * - Di0 : Control Button
 * 
 * 
 * THIS DESIGN IS A PROTOTYPE. THERE ARE NO GUARANTIES IT WILL WORK PROPERLY.
 * 
 * **************************************************************************************************************/



using System;
using Microsoft.SPOT;
using System.Threading;
using Microsoft.SPOT.Hardware;
using GHIElectronics.NETMF.FEZ;
using GHIElectronics.NETMF.Hardware;



public class Program
{

    const int DURATION = 480; // 480;                         // The duration of the action.
    const int UNLOAD_CHECK_INTERVAL = 15;
    const int WAIT_500ms = 500;



    static String[] identifiers =  new String[4];       // A string to identify each battery. It will be yyyy-mm-dd hh-nn - 0 (where '0' is the number of the channel)

    static int[][] loadedResults = new int[4][];
    static int[][] unLoadedResults = new int[4][];



    public static String TD(int numberToCheck)
    {
        String output = "" + numberToCheck;
        if (output.Length < 2) output = "0" + output;
        return output;

    }


    #region "test"
    public static void testSub() {
        AccuTest device = new AccuTest();
        AnalogIn BatteryVoltage = new AnalogIn((AnalogIn.Pin)FEZ_Pin.AnalogIn.An2);
        BatteryVoltage.SetLinearScale(0, 3300);

        device.closeAllChannels();


        int min=3300, max=0;

        while (true) {
            String test = "";

            for (int i = 0; i < 4; i++) 
            {
                device.selectMonitoredChannel (i);
                Thread.Sleep(100);
                BatteryVoltage.Read();

                int num = BatteryVoltage.Read();


                if (num < min) min = num;
                if (num > max) max = num;

                if (test == "")
                    test = "" + num;
                else
                    test = test + "," + num;


            }

            Debug.Print (test + " max=" + max + " min=" + min);
            Thread.Sleep(100);


        }
    }
    #endregion

    public static void Main()
    {

        //testSub();

        // Define the buttons of the program.
        //OutputPort statusLED = new OutputPort((Cpu.Pin)FEZ_Pin.Digital.Di1, false);     // green status LED
        PWM statusLED = new PWM((PWM.Pin)FEZ_Pin.PWM.Di5);
        PWM buzzer = new PWM((PWM.Pin)FEZ_Pin.PWM.Di8);
        InputPort button = new InputPort((Cpu.Pin)FEZ_Pin.Digital.Di0, false, Port.ResistorMode.PullDown);
        OutputPort LED = new OutputPort((Cpu.Pin)FEZ_Pin.Digital.LED, false);



        SD sdcard;
        try
        {
            sdcard = new SD();
        }
        catch (Exception e)
        {
            Debug.Print ("serious error! Program has to be reset....");
            buzzer.Set(1000, 50);
            while (true) ;
        }

        // Get the time from the RTC clock
        Utility.SetLocalTime(RealTimeClock.GetTime());

        // Define the accutest device.
        AccuTest device = new AccuTest();

        // define the array results (it's a Jagged Array as .NET micro Framework doesn't support multi dimensional arrays).
        loadedResults[0] = new int[DURATION];
        loadedResults[1] = new int[DURATION];
        loadedResults[2] = new int[DURATION];
        loadedResults[3] = new int[DURATION];
        unLoadedResults[0] = new int[32];
        unLoadedResults[1] = new int[32];
        unLoadedResults[2] = new int[32];
        unLoadedResults[3] = new int[32];



        // Define the analog input
        AnalogIn BatteryVoltage = new AnalogIn((AnalogIn.Pin)FEZ_Pin.AnalogIn.An2);
        BatteryVoltage.SetLinearScale(0, 3300);


        Debug.Print("Starting Battery Voltage Tester");
        Debug.Print("Copyright (c) 2011   Vanderhaegen Bart");
        Debug.Print("");

        buzzer.Set(0, 50);

        while (true)
        {

            // Here the program begins. 
            Debug.Print("Press the button to initiate a test...");
            statusLED.Set(4, 50);
            int activeChannel = -1;

            Boolean error = true;
            while (error)
            {
                // wait until the button is pressed.
                while (button.Read()) ;

                while (!button.Read()) ;    // TEST
                activeChannel++;
                if (activeChannel == 4) activeChannel = 0;

                // Try to mount the SD card
                try
                {
                    sdcard.mountCard();
                    error = false;
                    Debug.Print("SD card mounting successfully");
                }
                catch (Exception e)
                {
                    error = true;
                    Debug.Print("SD card mounting failed: " + e.Message);

                    buzzer.Set(2000, 50);
                    Thread.Sleep(1000);
                    buzzer.Set(0, 50);
                    // wait until the button is released.
                    while (!button.Read()) ;
                }
            }

            //RealTimeClock.SetTime(new DateTime(2011, 08, 15, 18, 30, 0, 0));



            // start the process light.
            statusLED.Set(1, 50);

            // Build the identifier Strings. They will be used for the filenames.
            String timeStamp = DateTime.Now.Year  + "." + 
                               TD(DateTime.Now.Month) + "." + 
                               TD(DateTime.Now.Day) + " " + 
                               TD(DateTime.Now.Hour) + "." + 
                               TD(DateTime.Now.Minute) + " #";

            identifiers[0] = timeStamp + "1";
            identifiers[1] = timeStamp + "2";
            identifiers[2] = timeStamp + "3";
            identifiers[3] = timeStamp + "4";
            Debug.Print(identifiers[0]);
            Debug.Print(identifiers[1]);
            Debug.Print(identifiers[2]);
            Debug.Print(identifiers[3]);





            Debug.Print("Phase 1: determing active channels");

            device.closeAllChannels();
            bool[] enabledChannels = new bool[4];

            // Phase 1: determine which accu's are inserted.
            for (int i = 0; i < 4; i++)
            {
                device.selectMonitoredChannel(i);
                Thread.Sleep(WAIT_500ms/2);
                int voltage = BatteryVoltage.Read();

                if (voltage > 400)
                {
                    Debug.Print((i + 1) + ": ACTIVE");
                    enabledChannels[i] = true;
                }
                else
                {
                    Debug.Print((i + 1) + ": INACTIVE");
                    enabledChannels[i] = false;
                }
            }



            long startTime, stopTime;
            startTime = DateTime.Now.Ticks;

            Debug.Print("Phase 2: testing battery loads");
            device.openAllChannels();
            bool extendedTestPerformed = false;



            //statusLED.Set(1, 50);
            int unloadedPassCounter = 0;

            int[] averageValues=new int[4];




            //
            // PHASE 2: THE ACTUAL TESTING PHASE
            //
            for (int minutes = 0; minutes < DURATION; minutes++)
            {
                // SET LED SPEED
                int percentage = (8 * minutes / DURATION);
                statusLED.Set(1 + percentage, 50);

                // RESET ARRAY
                for (int avgReset = 0; avgReset < 4; avgReset++)
                    averageValues[avgReset] = 0;


                // MEASURE FOR 50 SECONDS
                for (int secCounter = 0; secCounter < 25; secCounter++)     // 50 SECONDEN
                { 
                    for (int i = 0; i < 4; i++)                             // 2 SECONDEN
                    {
                        device.selectMonitoredChannel(i);
                        Thread.Sleep(WAIT_500ms);
                        int loadedV = BatteryVoltage.Read();
                        averageValues[i] += loadedV;
                    }
                }
                Debug.Print("tussentijd (norm 50 sec): " + ((DateTime.Now.Ticks - startTime) / 10000000));
                // CALCULATE AVERAGE RESULTS
                String summaryLoaded = "";
                for (int i = 0; i < 4; i++)
                {
                    loadedResults[i][minutes] = averageValues[i] / 25;
                    if (summaryLoaded == "")
                        summaryLoaded = "" + loadedResults[i][minutes];
                    else
                        summaryLoaded = summaryLoaded + "," + loadedResults[i][minutes]; 
                }
                Debug.Print("U (load=1): " + summaryLoaded);


                #region "test"
                //// Each minute a global check should be made.
                //// GUARANTEED TIME: 20 SECONDS.

                //for (int i = 0; i < 4; i++)
                //{
                //    device.selectMonitoredChannel(i);
                //    Thread.Sleep(2 * oneSecondIs);
                //    int loadedVoltage = BatteryVoltage.Read();
                //    Thread.Sleep(2 * oneSecondIs);
                //    if (summaryLoaded == "")
                //        summaryLoaded = "" + loadedVoltage;
                //    else
                //        summaryLoaded = summaryLoaded + "," + loadedVoltage; 

                //    loadedResults[i][minutes] = loadedVoltage;
                //}

                //Debug.Print("U (load=1): " + summaryLoaded);
                #endregion


                // Repeat an extended test every 15 minutes.
                if (minutes % 15 == 0)
                {
                    device.closeAllChannels();

                    String summaryUnLoaded = "";
                    extendedTestPerformed = true;

                    for (int i = 0; i < 4; i++)     // 10 SECONDS
                    {
                        device.selectMonitoredChannel(i);
                        Thread.Sleep(5 * WAIT_500ms);                   // 2.5 SECONDS
                        int unloadedVoltage = BatteryVoltage.Read(); 


                        if (summaryUnLoaded == "")
                            summaryUnLoaded = "" + unloadedVoltage;
                        else
                            summaryUnLoaded = summaryUnLoaded + "," + unloadedVoltage;

                        
                        unLoadedResults[i][unloadedPassCounter] = unloadedVoltage;
                    }

                    Debug.Print("**************** UNLOADED DATA ****************");
                    Debug.Print("U (load=0): " + summaryUnLoaded);
                    unloadedPassCounter++;
                    device.openAllChannels();
                    Debug.Print("tussentijd ext (norm 10 sec): " + ((DateTime.Now.Ticks - startTime) / 10000000));
                }


                if (extendedTestPerformed==false)
                {
                    // THREAD MUST NOT SLEEP: IT ALREADY TAKES 10 SEC
                    // SLEEP 10 SECONDS
                    
                    Thread.Sleep(20 * WAIT_500ms);
                    Debug.Print("tussentijd geen ext (norm 10 sec): " + ((DateTime.Now.Ticks - startTime) / 10000000));
                }
                else {
                    extendedTestPerformed = false;
                }            
            }
            

            statusLED.Set(1, 100);
            
            stopTime = DateTime.Now.Ticks;
            long duration;
            duration = ((DateTime.Now.Ticks - startTime) / 10000000) ;
            Debug.Print("duration of the operation is: " + duration + " seconds");


            // Phase 3: write the data to the microSD card!
            Debug.Print("Phase 3: Testing completed. Writing data to the SD cards...");





            for (int i = 0; i < 4; i++)
            {
                // WRITE THE FILE WITH THE LOADED DATA.
                if (enabledChannels[i] == true)
                {
                    sdcard.openFile(identifiers[i] + " - loaded.csv");
                    Debug.Print ("@ SD:\\" + identifiers[i] + " - loaded.csv");

                    // Write the loaded Data to the file
                    for (int j = 0; j < DURATION; j++)
                        sdcard.writeFile(j + ";" + loadedResults[i][j] + "\n");

                    sdcard.closeFile();

                    sdcard.openFile(identifiers[i] + " - unLoaded.csv");
                    Debug.Print("@ SD:\\" + identifiers[i] + " - unLoaded.csv");

                    // Write the loaded Data to the file
                    for (int j = 0; j < 32; j++)
                        sdcard.writeFile(j + ";" + unLoadedResults[i][j] + "\n");

                    sdcard.closeFile();
                }
            }

            


            for (int i = 0; i < 10; i++)
            {
                buzzer.Set(4000, 50);
                Thread.Sleep(500);
                buzzer.Set(0, 50);
                Thread.Sleep(500);
            }

            sdcard.unMountCard();


            Debug.Print("program terminated! Restarting after you press the button...");
            while (button.Read()) ;
            while (!button.Read()) ;
        }


    }
}